package soc;

/**
 * An implementation of the Forest-fire model described by Drossel and Schwabl
 * (1992). This implementation uses a 2-dimensional regular grid of cells where
 * each cell may be in one of three states: (1) empty, (2) occupied by a tree,
 * (3) burning. The state of each cell of the grid evolves over discrete time
 * steps according to four rules:
 * 
 * <ol>
 * <li>a burning cell becomes an empty cell
 * <li>a cell occupied by a tree becomes burning if at least one cell adjacent
 * to the tree is burning
 * <li>a cell occupied by a tree becomes burning with some probability even if
 * it has no burning cells adjacent to it
 * <li>an empty cell becomes occupied by a tree with some probability
 * </ol>
 * 
 * All cells are updated simultaneously at each time step.
 * 
 * <p>
 * Drossel B. and Schwabl F. (1992). Self-organized critical forest-fire model.
 * Physical Review Letters. Americal Physical Society. 69(11), 1629-1632.
 * 
 */
public class ForestFire {

	/**
	 * The probability that a tree ignites (becomes burning) during one time step.
	 */
	public static final double PROB_IGNITION = 0.000001;

	/**
	 * The probability that a tree grows in an empty cell during one time step.
	 */
	public static final double PROB_GROWTH = 0.0001;


	// FILL IN THE NEXT FOUR METHODS
	
	
	/**
	 * Returns the number of rows in the two-dimensional array of cells.
	 * 
	 * @param cells a two-dimensional array
	 * @return the number of rows in the two-dimensional array {@code cells}
	 */
	public static int rowsIn(State[][] cells) {

	
	}
	

	/**
	 * Returns the number of columns in the two-dimensional array of cells.
	 * 
	 * @param cells a two-dimensional array
	 * @return the number of columns in the two-dimensional array {@code cells}
	 */
	public static int colsIn(State[][] cells) {
		
	
	}

	/**
	 * Returns true if {@code row} is a valid row index and {@code col} is a valid
	 * column index for the two-dimensional array of cells, and false otherwise.
	 * 
	 * @param cells a two-dimensional array
	 * @param row   a row index
	 * @param col   a column index
	 * @return true if row and col are valid indexes for {@code cells}, false
	 *         otherwise
	 */
	public static boolean indexOk(State[][] cells, int row, int col) {
		
		
	}

	/**
	 * Prints the cells of a Forest-fire model followed by a new line.
	 * 
	 * <p>
	 * Empty cells are represented with a hyphen '-', trees are represented with the
	 * character 'T', and burning trees are represented with the character '#'.
	 * There are no separator characters between cells. Each row of {@code cells} is
	 * printed on a separate line.
	 * 
	 * @param cells a two-dimensional array
	 */
	public static void printForest(State[][] cells) {
		
		
	}


	// ADD THE FOUR REMAINING METHODS HERE
	// DOCUMENTATATION COMMENTS ARE NOT REQUIRED
	// MAKE SURE TO USE JAVA NAMING CONVENTIONS FOR THE METHOD NAMES

}
